package a2_withMultipleViews;

import java.awt.Color;
import java.awt.Point;
import java.util.EventListener;
import java.util.List;
import java.util.Vector;

/**
 * Encapsulation of LineDrawModel, taken from:
 * 
 * - Olsen Fig 2.5, p. 19 (called DrawModel in that figure)
 * 
 * - Fig 3.15, p. 67
 * 
 * - Fig 3.17 p. 68
 * 
 * MB: notice that this model does provide for the possibility for it to have
 * _listeners_ installed on it (such as those that a view might have so that it
 * can redraw itself in response to model changed events)
 * 
 * 
 * @author mb
 * 
 */
public class LineDrawModel {

	private List<Line> linesToDraw;

	public static int[] THICKNESS_VALUES = { 2, 4, 6, 8 };

	public static Color[] COLOUR_VALUES = { Color.BLACK, Color.RED, Color.GREEN };

	private List<LineDrawListener> listeners;

	public LineDrawModel() {
		linesToDraw = new Vector<Line>();
		listeners = new Vector<LineDrawListener>();
	}

	/**
	 * Adds the line specified by the passed x,y endpoint coordinates and
	 * returns it index
	 * 
	 * @param x1
	 * @param y1
	 * @param x2
	 * @param y2
	 * @return
	 */
	public int addLine(int x1, int y1, int x2, int y2) {
		Line line = new Line(x1, y1, x2, y2);
		linesToDraw.add(line);
		notifyModelHasChanged();
		return linesToDraw.size() - 1;
	}

	public int getNumberOfLines() {
		return linesToDraw.size();
	}

	public Point getEndPoint2(int index) {
		return linesToDraw.get(index).getEndPoint();
	}

	public Point getEndPoint1(int index) {
		return linesToDraw.get(index).getStartPoint();
	}

	public void addListener(LineDrawListener newListener) {
		listeners.add(newListener);
	}

	public void removeListener(LineDrawListener newListener) {
		listeners.remove(newListener);
	}

	private void notifyLineWillChange(int index, int newX1, int newY1,
			int newX2, int newY2) {
		for (LineDrawListener listener : listeners) {
			listener.lineWillChange(index, newX1, newY1, newX2, newY2);
		}
	}

	public void notifyModelHasChanged() {
		for (LineDrawListener listener : listeners) {
			listener.modelHasChanged();
		}
	}

	public Line getLine(int index) {
		return linesToDraw.get(index);
	}

	public Integer nearestLineByIndex(Point point) {
		Integer idx = null;
		for (int i = 0; i < getNumberOfLines(); i++) {
			if (getLine(i).isNearTo(point))
				idx = i;
		}
		return idx;
	}

	public Line nearestLine(Point point) {
		Integer idx = nearestLineByIndex(point);
		if (idx != null) {
			return getLine(idx);
		} else {
			return null;
		}
	}

	public void moveLine(int currentIndex, Point displacement) {
		Line theLine = linesToDraw.get(currentIndex);
		System.out.println("moveLine: " + theLine);
		theLine.displace(displacement);
		System.out.println("moveLine: " + theLine);
		notifyModelHasChanged();
	}

	public void advanceLineThicknessToNextValue(int currentIndex) {
		Line theLine = linesToDraw.get(currentIndex);
		theLine.advanceLineThicknessToNextValue();
		notifyModelHasChanged();
	}
	
	public void advanceLineColorToNextValue(int currentIndex) {
		Line theLine = linesToDraw.get(currentIndex);
		theLine.advanceColourToNextValue();
		notifyModelHasChanged();
	}


}
